package com.aitrainer.api.controller.diet

import com.aitrainer.api.model.diet.Meal
import com.aitrainer.api.repository.diet.MealRepository
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("api")
class MealController(private val mealRepository: MealRepository) {

    @PostMapping ("/meal", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun insert(@RequestBody meal: Meal): ResponseEntity<Meal> {
        val newMeal = mealRepository.save(meal)
        return ResponseEntity.ok().body(newMeal)
    }

    @GetMapping("/meal", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun getAll(): ResponseEntity<List<Meal>> {
        val list = mealRepository.findAll()
        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }

    @GetMapping("/meal/{id}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun getById(@PathVariable id: Long): ResponseEntity<Meal> {
        val rawMaterial = mealRepository.findById(id)
        return if (rawMaterial == null) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(rawMaterial)
    }

    @GetMapping("/meal/by_name/{name}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun getByName(@PathVariable name: String): ResponseEntity<Meal> {
        val meal = mealRepository.findByName(name)
        return if (meal == null) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(meal)
    }

    @PostMapping("/meal/{id}")
    fun update(@PathVariable(value = "id") id: Long, @RequestBody meal: Meal): ResponseEntity<Meal> {
        val existingMeal = mealRepository.findById(id) ?: return ResponseEntity.notFound().build()

        val updatedMeal: Meal = existingMeal.copy(
            id = existingMeal.id,
            name = existingMeal.name,
            normalizedName = meal.normalizedName,
            description = meal.description,
            calMin =  meal.calMin,
            calMax = meal.calMax,
            chMin = meal.chMin,
            chMax = meal.chMax,
            proteinMin = meal.proteinMin,
            proteinMax = meal.proteinMax,
            fatMin = meal.fatMin,
            fatMax = meal.fatMax,
            sugar = meal.sugar,
            serving = meal.serving,
            servingUnit = meal.servingUnit
        )

        return ResponseEntity.ok().body(mealRepository.save(updatedMeal))
    }

}